using HarmonyLib;
using System.Collections.Generic;
using System.Reflection;
using UnityEngine;

public class ConfigMachines : Mod
{
    static public JsonModInfo modInfo;
    Harmony harmony;

    private static Dictionary<CookingSlot, float> advGrillSlotMults = new Dictionary<CookingSlot, float>();
    private static Dictionary<CookingSlot, float> advPurifSlotMults = new Dictionary<CookingSlot, float>();
    private static Dictionary<CookingSlot, float> baseGrillSlotMults = new Dictionary<CookingSlot, float>();
    private static Dictionary<CookingSlot, float> basePurifSlotMults = new Dictionary<CookingSlot, float>();
    private static Dictionary<CookingSlot, float> elecGrillSlotMults = new Dictionary<CookingSlot, float>();
    private static Dictionary<CookingSlot, float> elecSmeltSlotMults = new Dictionary<CookingSlot, float>();
    private static Dictionary<CookingSlot, float> paintMillSlotMults = new Dictionary<CookingSlot, float>();
    private static Dictionary<CookingSlot, float> smelterSlotMults = new Dictionary<CookingSlot, float>();

    private static List<Dictionary<CookingSlot, float>> allCookingSlotDicts = new List<Dictionary<CookingSlot, float>>() {
        advGrillSlotMults,
        advPurifSlotMults,
        baseGrillSlotMults,
        basePurifSlotMults,
        elecGrillSlotMults,
        elecSmeltSlotMults,
        paintMillSlotMults,
        smelterSlotMults,
    };

    public void Start()
    {
        ConfigMachines.modInfo = modlistEntry.jsonmodinfo;
        this.harmony = new Harmony("com.destruction.ConfigurableMachines");
        this.harmony.PatchAll(Assembly.GetExecutingAssembly());

        // Find all the slots and add them to our update list.
        foreach (var slot in FindObjectsOfType<CookingSlot>())
        {
            // Only add if they are awake.
            if (slot.gameObject.activeInHierarchy)
            {
                ConfigMachines.AddCookingSlot(slot);
            }
        }

        ConfigMachines.Log("Mod has been loaded.");
    }

    public void OnModUnload()
    {
        ConfigMachines.RestoreMultipliers();
        this.harmony.UnpatchAll("com.destruction.ConfigurableMachines");
        ConfigMachines.Log("Mod has been unloaded.");
    }

    public static void Log(object message)
    {
        Debug.Log("[" + modInfo.name + "]: " + message.ToString());
    }

    public static void AddCookingSlot(CookingSlot slot)
    {
        // Get the original multiplier.
        float originalMult = (float)Traverse.Create(slot).Field("cookTimeMultiplier").GetValue();

        string name = slot.gameObject.name;

        // Now figure out where to put it.
        if (name.StartsWith("CookingSlot_Smelter_Electric"))
        {
            // Electric smelter slot.
            ConfigMachines.elecSmeltSlotMults.TryAdd(slot, originalMult);
            ConfigMachines.UpdateElectricSmelterSlot(slot);
        }
        else if (name.StartsWith("CookingSlot_Color"))
        {
            // Paint mill slot.
            ConfigMachines.paintMillSlotMults.TryAdd(slot, originalMult);
            ConfigMachines.UpdatePaintMillSlot(slot);
        }
        else if (name.StartsWith("CookingSlot_Food_BigGrill"))
        {
            if (slot.GetComponentInParent<CookingStand_Electric>())
            {
                // Electric grill slot.
                ConfigMachines.elecGrillSlotMults.TryAdd(slot, originalMult);
                ConfigMachines.UpdateElectricGrillSlot(slot);
            }
            else
            {
                // Advanced grill slot.
                ConfigMachines.advGrillSlotMults.TryAdd(slot, originalMult);
                ConfigMachines.UpdateAdvancedGrillSlot(slot);
            }
        }
        else if (name.StartsWith("CookingSlot_Food"))
        {
            // Basic grill slot.
            ConfigMachines.baseGrillSlotMults.TryAdd(slot, originalMult);
            ConfigMachines.UpdateBasicGrillSlot(slot);
        }
        else if (name.StartsWith("CookingSlot_Smelter"))
        {
            // Basic smelter.
            ConfigMachines.smelterSlotMults.TryAdd(slot, originalMult);
            ConfigMachines.UpdateSmelterSlot(slot);
        }
        else if (slot.gameObject.GetComponentInParent<Block_CookingStand_Purifier>())
        {
            name = slot.gameObject.GetComponentInParent<Block_CookingStand_Purifier>().gameObject.name;
            if (name.StartsWith("Placeable_CookingStand_Purifier_One"))
            {
                // Basic purifier.
                ConfigMachines.basePurifSlotMults.TryAdd(slot, originalMult);
                ConfigMachines.UpdateBasicPurifierSlot(slot);
            }
            else if (name.StartsWith("Placeable_CookingStand_Purifier_Two"))
            {
                // Advanced purifier.
                ConfigMachines.advPurifSlotMults.TryAdd(slot, originalMult);
                ConfigMachines.UpdateAdvancedPurifierSlot(slot);
            }
        }
    }

    public static void RestoreMultipliers()
    {
        // Restore all multipliers to their original values.
        foreach(var cookingSlotDict in ConfigMachines.allCookingSlotDicts)
        {
            cookingSlotDict.Clean();
            foreach (var slot in cookingSlotDict.Keys)
            {
                Traverse.Create(slot).Field("cookTimeMultiplier").SetValue(cookingSlotDict[slot]);
            }
        }
    }

    private static void UpdateAllSlots()
    {
        ConfigMachines.advGrillSlotMults.Clean();
        foreach(CookingSlot k in ConfigMachines.advGrillSlotMults.Keys)
        {
            ConfigMachines.UpdateAdvancedGrillSlot(k);
        }

        ConfigMachines.advPurifSlotMults.Clean();
        foreach(CookingSlot k in ConfigMachines.advPurifSlotMults.Keys)
        {
            ConfigMachines.UpdateAdvancedPurifierSlot(k);
        }

        ConfigMachines.baseGrillSlotMults.Clean();
        foreach(CookingSlot k in ConfigMachines.baseGrillSlotMults.Keys)
        {
            ConfigMachines.UpdateBasicGrillSlot(k);
        }

        ConfigMachines.elecGrillSlotMults.Clean();
        foreach(CookingSlot k in ConfigMachines.elecGrillSlotMults.Keys)
        {
            ConfigMachines.UpdateElectricGrillSlot(k);
        }

        ConfigMachines.elecSmeltSlotMults.Clean();
        foreach(CookingSlot k in ConfigMachines.elecSmeltSlotMults.Keys)
        {
            ConfigMachines.UpdateElectricSmelterSlot(k);
        }

        ConfigMachines.paintMillSlotMults.Clean();
        foreach(CookingSlot k in ConfigMachines.paintMillSlotMults.Keys)
        {
            ConfigMachines.UpdatePaintMillSlot(k);
        }

        ConfigMachines.basePurifSlotMults.Clean();
        foreach(CookingSlot k in ConfigMachines.basePurifSlotMults.Keys)
        {
            ConfigMachines.UpdateBasicPurifierSlot(k);
        }

        ConfigMachines.smelterSlotMults.Clean();
        foreach(CookingSlot k in ConfigMachines.smelterSlotMults.Keys)
        {
            ConfigMachines.UpdateSmelterSlot(k);
        }
    }

    private static void UpdateAdvancedGrillSlot(CookingSlot slot)
    {
        var field = Traverse.Create(slot).Field("cookTimeMultiplier");
        field.SetValue(ConfigMachines.advGrillSlotMults[slot] * ConfigMachines.GetAdvGrillMult());
    }

    private static void UpdateAdvancedPurifierSlot(CookingSlot slot)
    {
        var field = Traverse.Create(slot).Field("cookTimeMultiplier");
        field.SetValue(ConfigMachines.advPurifSlotMults[slot] * ConfigMachines.GetAdvPurifierMult());
    }

    private static void UpdateBasicGrillSlot(CookingSlot slot)
    {
        var field = Traverse.Create(slot).Field("cookTimeMultiplier");
        field.SetValue(ConfigMachines.baseGrillSlotMults[slot] * ConfigMachines.GetBaseGrillMult());
    }

    private static void UpdateBasicPurifierSlot(CookingSlot slot)
    {
        var field = Traverse.Create(slot).Field("cookTimeMultiplier");
        field.SetValue(ConfigMachines.basePurifSlotMults[slot] * ConfigMachines.GetBasePurifierMult());
    }

    private static void UpdateElectricGrillSlot(CookingSlot slot)
    {
        var field = Traverse.Create(slot).Field("cookTimeMultiplier");
        field.SetValue(ConfigMachines.elecGrillSlotMults[slot] * ConfigMachines.GetElecGrillMult());
    }

    private static void UpdateElectricSmelterSlot(CookingSlot slot)
    {
        var field = Traverse.Create(slot).Field("cookTimeMultiplier");
        field.SetValue(ConfigMachines.elecSmeltSlotMults[slot] * ConfigMachines.GetElecSmelterMult());
    }

    private static void UpdatePaintMillSlot(CookingSlot slot)
    {
        var field = Traverse.Create(slot).Field("cookTimeMultiplier");
        field.SetValue(ConfigMachines.paintMillSlotMults[slot] * ConfigMachines.GetPaintMillMult());
    }

    private static void UpdateSmelterSlot(CookingSlot slot)
    {
        var field = Traverse.Create(slot).Field("cookTimeMultiplier");
        field.SetValue(ConfigMachines.smelterSlotMults[slot] * ConfigMachines.GetSmelterMult());
    }

    public static float GetAdvPurifierMult()
    {
        return ExtraSettingsAPI_GetSliderValue("advancedPurifierMultiplier");
    }

    public static float GetAdvGrillMult()
    {
        return ExtraSettingsAPI_GetSliderValue("advancedSmelterMultiplier");
    }

    public static float GetBaseGrillMult()
    {
        return ExtraSettingsAPI_GetSliderValue("basicGrillMultiplier");
    }

    public static float GetBasePurifierMult()
    {
        return ExtraSettingsAPI_GetSliderValue("basicPurifierMultiplier");
    }

    public static float GetElecGrillMult()
    {
        return ExtraSettingsAPI_GetSliderValue("electricGrillMultiplier");
    }

    public static float GetElecSmelterMult()
    {
        return ExtraSettingsAPI_GetSliderValue("electricSmelterMultiplier");
    }

    public static float GetPaintMillMult()
    {
        return ExtraSettingsAPI_GetSliderValue("paintMillMultiplier");
    }

    public static float GetSmelterMult()
    {
        return ExtraSettingsAPI_GetSliderValue("basicSmelterMultiplier");
    }

    // ExtraSettingsAPI stuff.
    public void ExtraSettingsAPI_Load() // Occurs when the API loads the mod's settings
    {
        ConfigMachines.UpdateAllSlots();
    }

    public void ExtraSettingsAPI_SettingsClose() // Occurs when the API loads the mod's settings
    {
        ConfigMachines.UpdateAllSlots();
    }

    public static float ExtraSettingsAPI_GetSliderValue(string SettingName) => 1f;

    [HarmonyPatch(typeof(CookingSlot), "Awake")]
    public class Patch_CookingSlotAwake
    {
        static void Postfix(ref CookingSlot __instance)
        {
            // On awake, add to the dictionary. This will update it.
            ConfigMachines.AddCookingSlot(__instance);
        }
    }
}



static class DictionaryExtension
{
    // Thanks Aidan.
    public static void Clean<X,Y>(this Dictionary<X,Y> c) where X : MonoBehaviour
    {
        var l = new List<X>();
        foreach (var k in c.Keys)
        {
            if (!k)
            {
                l.Add(k);
            }
        }
        foreach (var i in l)
        {
            c.Remove(i);
        }
    }
}
